package uoc.tfc.clientbs;

import uoc.tfc.database.Database;
import uoc.tfc.entities.TakeItem;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.ActionBarActivity;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.TextView;

/**
 * Activity executed when an Alarm is produced.
 * 
 * @author omnizeus
 *
 */
public class AlarmActivity extends ActionBarActivity {

	// Activity Controls
	TextView tvProduct = null;
	TextView tvQuantity = null;
	TextView tvId = null;

	// Database elements
	private Database db;

	// Current TakeItem data
	private int requestCode;
	private long takeItemId;
	private String takeItemProduct;
	private int takeItemQuantity;

	/**
	 * On Create event.
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_alarm);
		String whereami = ":onCreate";

		try {
			Window win;
			Log.i(this.getClass().getName() + whereami, this.takeItemId + "; "
					+ this.takeItemProduct + "; " + this.takeItemQuantity);

			/**
			 * Window activity configuration.
			 */
			win = this.getWindow();
			win.addFlags(WindowManager.LayoutParams.FLAG_DISMISS_KEYGUARD);
			win.addFlags(WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED);
			win.addFlags(WindowManager.LayoutParams.FLAG_TURN_SCREEN_ON);
			// Force screen always on & fullscreen mode.
			win.addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
			win.addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);

			/**
			 * Getting activity controls.
			 */
			this.tvProduct = (TextView) findViewById(R.id.tv_product);
			this.tvQuantity = (TextView) findViewById(R.id.tv_quantity);
			this.tvId = (TextView) findViewById(R.id.tv_id);

			this.requestCode = this.getIntent().getExtras()
					.getInt("requestCode");
			this.takeItemId = this.getIntent().getExtras().getLong("id");
			this.takeItemProduct = this.getIntent().getExtras()
					.getString("product");
			this.takeItemQuantity = this.getIntent().getExtras()
					.getInt("quantity");
			this.tvProduct.setText(this.takeItemProduct);
			this.tvQuantity.setText(String.valueOf(this.takeItemQuantity));

			/**
			 * Prepare database
			 */
			db = new Database(getApplicationContext());

			/**
			 * Get only one take list item and update activity info.
			 */
			TakeItem takeitem = db.GetOneTakeItem(this.requestCode);
			if (takeitem != null) {
				this.updateProductInfo(takeitem);
			}
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}

	}

	/**
	 * On resume activity.
	 */
	@Override
	protected void onPostResume() {
		super.onPostResume();
		String whereami = ":onPostResume";
		try {
			TakeItem takeitem = db.GetOneTakeItem(this.requestCode);
			if (takeitem != null) {
				this.updateProductInfo(takeitem);
			}
			Log.i(this.getClass().getName() + whereami, "init. takeItemId: "
					+ this.takeItemId);
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}
	}

	/**
	 * Button "Taken"
	 * 
	 * @param view
	 */
	public void btnTaken(View view) {
		String whereami = ":btnTaken";
		Log.i(this.getClass().getName() + whereami,
				"TakeItemId: " + String.valueOf(this.takeItemId));
		try {
			// Set that product was taken.
			db.FinishTakeItem(this.takeItemId);
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception(1)", e);
		}
		try {
			TakeItem takeitem = db.GetOneTakeItem(this.requestCode);
			if (takeitem != null) {
				// Only re-open this activity if there are more take items.
				Intent intent = getIntent();
				super.finish();
				super.startActivity(intent);
			} else {
				// Otherwise, close this activity.
				super.finish();
			}
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception(2)", e);
		}
	}

	/**
	 * Button "Not taken"
	 * 
	 * @param view
	 */
	public void btnNottaken(View view) {
		String whereami = ":btnNottaken";
		Log.i(this.getClass().getName() + whereami, "Close activity");
		super.finish();
	}

	/**
	 * Update the activity controls with Take Item data.
	 * 
	 * @param takeitem
	 */
	private void updateProductInfo(TakeItem takeitem) {
		String whereami = ":updateProductInfo";
		try {
			this.takeItemId = takeitem.getId();
			this.takeItemProduct = takeitem.getProduct();
			this.takeItemQuantity = takeitem.getQuantity();
			this.tvProduct.setText(takeitem.getProduct());
			this.tvQuantity.setText(String.valueOf(takeitem.getQuantity()));
			// For debug purposes only
			// this.tvId.setText(String.valueOf(takeitem.getId()));
			this.tvId.setText("");
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}
	}

}
