package uoc.tfc.clientbs;

import java.util.Timer;
import java.util.TimerTask;

import uoc.tfc.database.Database;
import uoc.tfc.entities.User;
import uoc.tfc.httpman.HttpHelper;
import uoc.tfc.httpman.HttpTaskMan;
import uoc.tfc.task.GlobalAsyncTaskListener;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.support.v7.app.ActionBarActivity;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

/**
 * Activity for full login process, including getting user data from BS Service.
 * 
 * @author omnizeus
 *
 */
public class LoginActivity extends ActionBarActivity {

	// Constants (fixed service username and password)
	private static String BSSERVICE_NICKNAME = "clientbs@dev00010";
	private static String BSSERVICE_PASSWORD = "Tfc2014#$";

	// Edit fields
	private static EditText editNickname;
	private static EditText editPassword;
	private static EditText editIdpatientph;
	private static EditText editClientbspassword;

	// Status controls
	static private TextView tvStatus;

	// Buttons
	private Button btnLogin;
	private Button btnCancel;

	// Database
	private Database db;

	// Internal flags
	private boolean userIsValid = false;
	private boolean userHasTreatments = false;

	/**
	 * On Create event. 
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		String whereami = ":onCreate";
		setContentView(R.layout.activity_login);
		boolean autologin = true;
		try {

			/**
			 * Check if activity is called in autologin mode.
			 */
			Intent myIntent = getIntent();
			if (myIntent.getStringExtra("autologin") != null)
				autologin = Boolean.valueOf(myIntent
						.getStringExtra("autologin"));

			/**
			 * Get view elements: edit texts and buttons.
			 */
			LoginActivity.editNickname = (EditText) findViewById(R.id.edit_nickname);
			LoginActivity.editPassword = (EditText) findViewById(R.id.edit_password);
			LoginActivity.editIdpatientph = (EditText) findViewById(R.id.edit_idpatientph);
			LoginActivity.editClientbspassword = (EditText) findViewById(R.id.edit_clientbspassword);
			tvStatus = (TextView) findViewById(R.id.tv_status);
			btnLogin = (Button) findViewById(R.id.btn_login);
			btnCancel = (Button) findViewById(R.id.btn_cancel);

			/**
			 * Prepare database
			 */
			db = new Database(getApplicationContext());

			/**
			 * Get stored user info
			 */
			User user = db.GetStoredUser();

			if (user != null) {
				
				/**
				 * If there is stored user, fill the view elements. NOTE: The
				 * listener is the full login process one.
				 */
				LoginActivity.editNickname.setText(user.getNickname());
				LoginActivity.editPassword.setText(user.getPassword());
				LoginActivity.editIdpatientph.setText(user.getIdpatientph());
				LoginActivity.editClientbspassword.setText(user
						.getClientbspassword());
				/**
				 * Autologin
				 */
				if (user.getIdpatientph() != null
						&& !user.getIdpatientph().equals("")) {
					if (autologin) {
						this.lockUI(true);
						// Autologin: agafa com a listener el "gestor de login".
						new HttpTaskMan(this, Global.getCookieStore(),
								Global.getHttpContext(),
								new FetchLoginProcessListener())
								.execute(HttpTaskMan.TASK_CHECK_BS_SERVICE_STATUS);
						}
				}
				
			} else {
				
				/**
				 * If there is not stored user, only check service status. NOTE:
				 * The listener is the basic one.
				 */
				new HttpTaskMan(this, Global.getCookieStore(),
						Global.getHttpContext(), new FetchHttpTaskListener())
						.execute(HttpTaskMan.TASK_CHECK_BS_SERVICE_STATUS);

			}

		} catch (Exception e) {

			Log.e(this.getClass().getName() + whereami, e.toString());

		}

	}

	/**
	 * Login button: starts the full login process.
	 * 
	 * @param view
	 */
	public void btnLogin(View view) {
		String whereami = ":btnLogin";
		try {
			/**
			 * Connecting message and lock the ui.
			 */
			setStatus(HttpHelper.RESP_SUCCESS_LOGIN_CONNECTING);
			this.lockUI(true);

			/**
			 * Start full login process.
			 */
			new HttpTaskMan(this, Global.getCookieStore(),
					Global.getHttpContext(), new FetchLoginProcessListener())
					.execute(HttpTaskMan.TASK_CHECK_BS_SERVICE_STATUS);

		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}

	}

	/**
	 * Logout button: exit app.
	 * 
	 * @param view
	 */
	public void btnLogout(View view) {
		super.finish();
	}

	/**
	 * Change the status message and color.
	 * 
	 * @param msg
	 */
	public static void setStatus(String msg) {
		String procmsg = "";
		int mode = 0;
		try {
			procmsg = msg.split("#")[1];
			mode = Integer.valueOf(msg.split("#")[0]);
			if (mode > 0) {
				tvStatus.setBackgroundColor(Color.parseColor("#66FF66"));
			} else if (mode == 0) {
				tvStatus.setBackgroundColor(Color.parseColor("#DDDDFF"));
			} else if (mode < 0) {
				tvStatus.setBackgroundColor(Color.parseColor("#FF6666"));
			}
			tvStatus.setText(procmsg);
		} catch (Exception e) {
			tvStatus.setBackgroundColor(Color.parseColor("#FF6666"));
			tvStatus.setText(e.toString());
		}
	}

	/**
	 * Get the message text from HttpResponse.
	 * 
	 * @param msg
	 * @return
	 */
	public int getStatus(String msg) {
		String whereami = ":getStatus";
		int mode = 0;
		try {
			mode = Integer.valueOf(msg.split("#")[0]);
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}
		return mode;
	}

	/**
	 * Lock/unlock the UI in order to avoid user manipulation.
	 * 
	 * @param lock
	 */
	public void lockUI(Boolean lock) {
		String whereami = ":lockUI";
		try {
			if (lock) {
				LoginActivity.editNickname.setEnabled(false);
				LoginActivity.editPassword.setEnabled(false);
				LoginActivity.editIdpatientph.setEnabled(false);
				LoginActivity.editClientbspassword.setEnabled(false);
				btnLogin.setEnabled(false);
				btnCancel.setEnabled(false);
			} else {
				LoginActivity.editNickname.setEnabled(true);
				LoginActivity.editPassword.setEnabled(true);
				LoginActivity.editIdpatientph.setEnabled(true);
				LoginActivity.editClientbspassword.setEnabled(true);
				btnLogin.setEnabled(true);
				btnCancel.setEnabled(true);
			}
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}

	}

	/**
	 * Go to main activity.
	 */
	private void goToMainActivity() {
		String whereami = ":goToMainActivity";
		try {
			TimerTask task = new TimerTask() {
				@Override
				public void run() {

					Intent mainIntent = new Intent().setClass(LoginActivity.this,
							MainActivity.class);

					startActivity(mainIntent);
					finish(); // Destroy this activity to avoid user return
								// with back button.
				}
			};
			Timer timer = new Timer();
			timer.schedule(task, 500); // Wait a little			
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}

	}

	/**
	 * Full login process.
	 *
	 */
	public class FetchLoginProcessListener implements
			GlobalAsyncTaskListener<String> {

		/**
		 * Manages the http tasks completion.
		 */
		@Override
		public void onTaskComplete(String result) {

			Log.i(getClass().getName(), "INIT: " + result);

			/**
			 * Shows the task response.
			 */
			setStatus(result);

			/**
			 * Response manager.
			 */
			if (result.equals(HttpHelper.RESP_SUCCESS_SERVICE_AVAILABLE)) {

				// Service is available

				Log.i(getClass().getName() + ":" + "FetchLoginProcessListener",
						HttpHelper.RESP_SUCCESS_SERVICE_AVAILABLE);

				// STEP 1: Service login (with system user and password)
				setStatus("0#Verificant usuari...");
				new HttpTaskMan(getApplicationContext(),
						Global.getCookieStore(), Global.getHttpContext(), this)
						.execute(HttpTaskMan.TASK_LOGIN, BSSERVICE_NICKNAME,
								BSSERVICE_PASSWORD);

			} else if (result.equals(HttpHelper.RESP_SUCCESS_LOGIN_CORRECT)) {

				// System login was correct.

				Log.i(getClass().getName() + ":" + "FetchLoginProcessListener",
						HttpHelper.RESP_SUCCESS_LOGIN_CORRECT);

				// STEP 2: Get user treatment/posology data.
				setStatus("0#Obtenint dades de l'usuari...");
				new HttpTaskMan(getApplicationContext(),
						Global.getCookieStore(), Global.getHttpContext(), this)
						.execute(HttpTaskMan.TASK_GET_REMOTE_POSOLOGY_DATA,
								LoginActivity.editIdpatientph.getText()
										.toString(),
								LoginActivity.editClientbspassword.getText()
										.toString());

			} else if (result.equals(HttpHelper.RESP_SUCCESS_PATIENT_DATA_OK)) {

				// User treatment/posology data is ok.
				userIsValid = true;
				userHasTreatments = true;
				Log.i(getClass().getName() + ":" + "FetchLoginProcessListener",
						HttpHelper.RESP_SUCCESS_PATIENT_DATA_OK);

				// STEP 3: System logout.
				userIsValid = true;
				new HttpTaskMan(getApplicationContext(),
						Global.getCookieStore(), Global.getHttpContext(), this)
						.execute(HttpTaskMan.TASK_LOGOUT);

			} else if (result.equals(HttpHelper.RESP_SUCCESS_PATIENT_NO_DATA)) {

				// User has no treatment data. System logout.

				userIsValid = true;
				new HttpTaskMan(getApplicationContext(),
						Global.getCookieStore(), Global.getHttpContext(), this)
						.execute(HttpTaskMan.TASK_LOGOUT);

			} else if (userIsValid && userHasTreatments
					&& result.equals(HttpHelper.RESP_SUCCESS_LOGOUT)) {

				// System is logged out.

				Log.i(getClass().getName() + ":" + "FetchLoginProcessListener",
						HttpHelper.RESP_SUCCESS_LOGOUT);

				// STEP 4: Starts main activity
				goToMainActivity();

			} else if (result.equals(HttpHelper.RESP_ERROR_PATIENT_DATA_ERR)) {

				// User is not correct. System logout.

				userIsValid = false;
				new HttpTaskMan(getApplicationContext(),
						Global.getCookieStore(), Global.getHttpContext(), this)
						.execute(HttpTaskMan.TASK_LOGOUT);

			} else if (result.equals(HttpHelper.RESP_SUCCESS_LOGOUT)) {

				if (!userIsValid) {
					// System is logout, and user is not correct.
					setStatus(getResources().getString(
							R.string.login_user_incorrect));
				} else if (!userHasTreatments) {
					// System is logout, and user is not correct.
					setStatus(getResources().getString(
							R.string.login_user_no_data));
				}
				// Enable UI.
				lockUI(false);

			} else if (result
					.equals(HttpHelper.RESP_ERROR_LOGIN_SERVICE_UNAVAILABLE)) {

				// Service is unavailable (wifi off, server off...). Get local
				// data

				// Check is local data is available!
				// TODO: ...

				// TODO: Get Local Data. If OK -> Main Activity
				goToMainActivity();

			} else {

				// Elsewhere, enable UI.

				lockUI(false);

			}
		}
	}

	/**
	 * Default listener for service check.
	 *
	 */
	public class FetchHttpTaskListener implements
			GlobalAsyncTaskListener<String> {

		@Override
		public void onTaskComplete(String result) {

			// Toast.makeText(getApplicationContext(), "SERVICE FINISH",
			// Toast.LENGTH_SHORT).show();
			Log.d("FetchHttpTaskLister", result);

			setStatus(result);
			if (getStatus(result) >= 0) {

				// Service available.

				// TODO: get local data!!
				// If local data, go to Main.

			} else {

				if (getStatus(result) == getStatus(HttpHelper.RESP_ERROR_LOGIN_INCORRECT)) {
					LoginActivity.editNickname.setText("");
					LoginActivity.editPassword.setText("");
					LoginActivity.editIdpatientph.setText("");
					LoginActivity.editClientbspassword.setText("");
				}
				lockUI(false);

			}
		}
	}

}
