package uoc.tfc.clientbs;

import java.util.Iterator;

import uoc.tfc.alarm.AlarmManagerBroadcastReceiver;
import uoc.tfc.database.Database;
import uoc.tfc.entities.HoraryEntry;
import uoc.tfc.ui.UIUtils;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.ActionBarActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.Toast;

/**
 * Activity for main process and user interaction. It has user menu (accessible
 * with Menu button).
 * 
 * @author omnizeus
 *
 */
public class MainActivity extends ActionBarActivity {
	// Broadcast receiver for getting alarms.
	private AlarmManagerBroadcastReceiver alarm;

	// Database
	private Database db;

	// UI utilities
	private UIUtils uiutils;

	/**
	 * On Create event.
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		String whereami = ":onCreate";
		try {
			/**
			 * Prepare the UIUtils.
			 */
			uiutils = new UIUtils(this);

			/**
			 * Prepare database.
			 */
			db = new Database(getApplicationContext());

			/**
			 * Create Alarm object.
			 */
			alarm = new AlarmManagerBroadcastReceiver();

			/**
			 * Enable the stored "horarylist" alarms.
			 */
			this.notifyAlarm(true);

		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}
	}

	/**
	 * Inflate the menu with main options.
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.main, menu);
		return super.onCreateOptionsMenu(menu);
	}

	/**
	 * Manage the option menu selection.
	 */
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// Handle action bar item clicks here. The action bar will
		// automatically handle clicks on the Home/Up button, so long
		// as you specify a parent activity in AndroidManifest.xml.
		String whereami = ":onOptionsItemSelected";
		try {
			int id = item.getItemId();
			if (id == R.id.action_login) {
				/**
				 * Open the "login/service access data" View.
				 */
				Intent loginActivity = new Intent(this, LoginActivity.class);
				loginActivity.putExtra("autologin", "false");
				startActivity(loginActivity);

			} else if (id == R.id.action_confighours) {
				/**
				 * Open the horary configuration View.
				 */
				Intent configActivity = new Intent(this, ConfigActivity.class);
				startActivity(configActivity);

			} else if (id == R.id.action_viewtreatments) {
				/**
				 * View treatment/posology data.
				 */
				Intent viewdataActivity = new Intent(this,
						ViewdataActivity.class);
				startActivity(viewdataActivity);

			} else if (id == R.id.action_enablealarms) {
				/**
				 * Enable all alarms.
				 */
				this.notifyAlarm(true);

			} else if (id == R.id.action_disablealarms) {
				/**
				 * Disable all alarms.
				 */
				this.notifyAlarm(false);
			}
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}
		return super.onOptionsItemSelected(item);
	}

	/**
	 * When back button is pressed the App is thrown to background. A message is
	 * shown with a notification.
	 */
	@Override
	public void onBackPressed() {
		String whereami = ":onBackPressed";
		// super.onBackPressed();
		try {
			Toast.makeText(this,
					this.getResources().getString(R.string.notif_taskToBack),
					Toast.LENGTH_LONG).show();
			Log.d(this.getClass().getName() + whereami,
					"Application to background.");
			this.moveTaskToBack(true);
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}
	}

	/**
	 * When the App comes to foreground it plays a sound.
	 */
	@Override
	protected void onRestart() {
		super.onRestart();
		String whereami = ":onRestart";
		try {
			Log.d(this.getClass().getName() + whereami,
					"Application to foreground.");
			uiutils.playSound(-1);
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}
	}

	/**
	 * After the App starts it plays a sound.
	 */
	@Override
	protected void onRestoreInstanceState(Bundle savedInstanceState) {
		super.onRestoreInstanceState(savedInstanceState);
		String whereami = ":onRestoreInstanceState";
		try {
			Log.d(this.getClass().getName() + whereami, "Application started.");
			uiutils.playSound(-1);
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);			
		}
	}
	
	/**
	 * When App returns to foreground. 
	 */
	@Override
	protected void onPostResume() {
		super.onPostResume();
		this.notifyAlarm(true);
	}

	/**
	 * Get the status code from a code#msg formatted text.
	 * 
	 * @param msg
	 *            Response formatted in code#msg style.
	 * @return
	 */
	public int getStatus(String msg) {
		int status = 0;
		String whereami = ":getStatus";
		try {
			status = Integer.valueOf(msg.split("#")[0]);
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}
		return status;
	}

	/**
	 * Enable/Disable alarms.
	 * 
	 * @param enable
	 *            Enable/Disable alarms.
	 */
	private void enableHoraryDataAlarms(boolean enable) {
		String whereami = ":enableHoraryDataAlarms";
		Iterator<HoraryEntry> horarylist = null;
		try {
			horarylist = db.GetHoraryList().iterator();
			while (horarylist.hasNext()) {
				HoraryEntry horary = horarylist.next();
				if (enable) {
					alarm.SetAlarm(this, horary.getToma(), horary.getHour(),
							horary.getMinute());
					Log.d(this.getClass().getName() + whereami,
							"Enabled Alarm ["
									+ horary.getToma()
									+ "] ("
									+ UIUtils.formatHValue(horary.getHour(),
											horary.getMinute()) + ").");
				} else {
					alarm.CancelAlarm(this, horary.getToma());
					Log.d(this.getClass().getName() + whereami,
							"Cancelled Alarm ["
									+ horary.getToma()
									+ "] ("
									+ UIUtils.formatHValue(horary.getHour(),
											horary.getMinute()) + ").");
				}
			}
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}
	}

	/**
	 * Shows a notification indicating that Alarms are on (or off).
	 * 
	 * @param enable
	 *            Notify Enable/Disable alarms.
	 */
	private void notifyAlarm(boolean enable) {
		String whereami = ":notifyAlarm";
		try {
			if (enable) {
				this.enableHoraryDataAlarms(true);
				Toast.makeText(this,
						this.getResources().getString(R.string.notif_alarmOn),
						Toast.LENGTH_SHORT).show();
			} else {
				this.enableHoraryDataAlarms(false);
				Toast.makeText(this,
						this.getResources().getString(R.string.notif_alarmOff),
						Toast.LENGTH_SHORT).show();
			}			
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}
	}

}
