package uoc.tfc.database;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Locale;

import org.json.JSONArray;
import org.json.JSONObject;

import uoc.tfc.entities.HoraryEntry;
import uoc.tfc.entities.TakeItem;
import uoc.tfc.entities.User;
import uoc.tfc.ui.UIUtils;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteStatement;
import android.util.Log;

/**
 * Class for managing local database connections.
 * 
 * @author omnizeus
 *
 */
public class Database {
	
	private DatabaseHelper dbhelper;
	private SQLiteDatabase database;
	Context context;
	
	/**
	 * Constructor.
	 * 
	 * @param _context
	 */
	public Database(Context _context) {
		this.context = _context;
		//this.dbhelper = new DatabaseHelper(context);
		this.dbhelper = DatabaseHelper.getInstance(context);
		this.database = dbhelper.getWritableDatabase();
		
		//this.dbhelper.onCreate(this.database); //FOR TESTING ONLY

	}

	/**
	 * Alternative method for storing user data into local storage.
	 * 
	 * @param nickname
	 * @param password
	 * @param idpatientph
	 * @param clientbspassword
	 * @return
	 */
	public boolean StoreUser(String nickname, String password, String idpatientph, String clientbspassword) {
		return this.StoreUser(new User(nickname, password, idpatientph, clientbspassword));
	}
	
	/**
	 * Store the user (an object with full service access data) into local storage.
	 * It will be used for autologin purposes.
	 * 
	 * @param user
	 * @return
	 */
	public boolean StoreUser(User user) {
		String whereami = ":storeUser";
		try {
			this.database.execSQL("DELETE FROM user;");
			this.database.execSQL("INSERT INTO user (nickname, password, idpatientph, clientbspassword) VALUES (?, ?, ?, ?);", new String[]{user.getNickname(), user.getPassword(), user.getIdpatientph(), user.getClientbspassword()});
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}
		return true;
	}
	
	/**
	 * Store the takelist (get from BS servlet) into local storage.
	 * 
	 * @param jarrData
	 * @return
	 */
	public boolean StoreTakelist(JSONArray jarrData) {
		try {
			//Log.d(this.getClass().getName(), jarrData.toString());
			this.database.execSQL("DELETE FROM takelist;");
			for(int i = 0; i < jarrData.length(); i++) {
				String takeitemdata = jarrData.getString(i);
				JSONObject takeitem = new JSONObject(takeitemdata);
				JSONArray productlist = new JSONArray(takeitem.getString("productlist"));
				for(int j = 0; j < productlist.length(); j++) {
					JSONObject productitem = new JSONObject(productlist.getString(0));
					
					SQLiteStatement insert = this.database.compileStatement("INSERT INTO takelist (date, toma, product, quantity) VALUES (?, ?, ?, ?);");

					Calendar calendar = Calendar.getInstance();
					calendar.setTimeInMillis(takeitem.getLong("date"));
					
					insert.bindString(1, new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime()));
					insert.bindLong(2, takeitem.getInt("toma"));
					insert.bindString(3, productitem.getString("product"));
					insert.bindLong(4, productitem.getInt("quantity"));
					insert.execute();
					
					Log.d(this.getClass().getName(), " - - - - " + new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime()) + " : " + productitem.getString("product"));
				}
			}
		} catch (Exception e) {
			Log.e(this.getClass().getName() + ":" + "storeTakelist", "json object?" , e);
		}
		return true;
	}
	
	/**
	 * Get the user stored in local storage.
	 * @return
	 */
	public User GetStoredUser() {
		Cursor cursor = null;
		User user = new User();
		try {
			cursor = this.database.rawQuery("SELECT nickname, password, idpatientph, clientbspassword FROM user;", new String[]{});
			cursor.moveToFirst();
			user = new User();
			user.setNickname(cursor.getString(0));
			user.setPassword(cursor.getString(1));
			user.setIdpatientph(cursor.getString(2));
			user.setClientbspassword(cursor.getString(3));
		} catch (SQLiteException e) {
			Log.e(this.getClass().getName(), "SQLiteException", e);
		} catch (Exception e) {
			Log.e(this.getClass().getName(), "Exception", e);
		} finally {
			try {
				if(cursor!=null) cursor.close(); cursor = null;
			} catch (Exception e2) {
				Log.e(this.getClass().getName(), "Exception", e2);				
			}
		}
		return user;
	}
	
	/*
	public String GetStoredTakeList3() {
		Cursor cursor = null;
		StringBuffer sb = null;
		try {
			sb = new StringBuffer();
			cursor = this.database.rawQuery("SELECT id, date, toma, product, quantity, status FROM takelist;", new String[]{});
			cursor.moveToFirst();
			while(!cursor.isAfterLast()) {
				sb.append(cursor.getString(0) + "; " + cursor.getString(1) + "; " + cursor.getString(2) + ";\n" +
							"\t" + cursor.getString(3) + ";\n" +
							"\t\t\t" + cursor.getString(4) + "; " + cursor.getString(5) + "\n\n");
				Log.d(this.getClass().getName() + ":" + "getStoredTakeList", cursor.getLong(0) + "; " + cursor.getString(1) + " ; " + cursor.getString(2) + " ; " + cursor.getString(3) + " ; " + cursor.getInt(4) + " ; " + cursor.getInt(5));
				cursor.moveToNext();
			}
		} catch (SQLiteException e) {
			Log.e(this.getClass().getName() + ":" + "getStoredTakeList", "SQLiteException", e);
		} catch (Exception e) {
			Log.e(this.getClass().getName() + ":" + "getStoredTakeList", "Exception", e);
		} finally {
			try {
				if (cursor!=null) cursor.close(); cursor = null;
			} catch (Exception e) {
				Log.e(this.getClass().getName() + ":" + "getStoredTakeList", "Exception(2)", e);
			}
		}
		return sb.toString();
	}
	*/
	
	/**
	 * Get the full take list. To Viewdata activity.
	 * @return
	 */
	public ArrayList<String> GetStoredTakeList() {
		Cursor cursor = null;
		ArrayList<String> takeList = new ArrayList<String>();
		try {
			cursor = this.database.rawQuery("SELECT id, date, toma, product, quantity, status FROM takelist;", new String[]{});
			cursor.moveToFirst();
			while(!cursor.isAfterLast()) {
				takeList.add("[" + cursor.getString(0) + "] " + cursor.getString(1) + " (" + cursor.getString(2) + ")\n" +
						"" + cursor.getString(3) + "\n" +
						"\t\t\tQuantitat: " + cursor.getString(4) + "; Estat: " + cursor.getString(5) + "");
				Log.d(this.getClass().getName() + ":" + "getStoredTakeList", cursor.getLong(0) + "; " + cursor.getString(1) + " ; " + cursor.getString(2) + " ; " + cursor.getString(3) + " ; " + cursor.getInt(4) + " ; " + cursor.getInt(5));
				cursor.moveToNext();
			}
		} catch (SQLiteException e) {
			Log.e(this.getClass().getName() + ":" + "getStoredTakeList", "SQLiteException", e);
		} catch (Exception e) {
			Log.e(this.getClass().getName() + ":" + "getStoredTakeList", "Exception", e);
		} finally {
			try {
				if (cursor!=null) cursor.close(); cursor = null;
			} catch (Exception e) {
				Log.e(this.getClass().getName() + ":" + "getStoredTakeList", "Exception(2)", e);
			}
		}
		return takeList;
	}
	
	/**
	 * Get the full horarylist from local storage. This is for config activity.
	 * @return
	 */
	public ArrayList<HoraryEntry> GetHoraryList() {
		String whereami = ":GetHoraryList";
		Cursor cursor = null;
		ArrayList<HoraryEntry> horarylist = null;
		try {
			Log.d(this.getClass().getName() + ":" + "getHoraryList", "Start!");
			horarylist = new ArrayList<HoraryEntry>();
			cursor = this.database.rawQuery("SELECT toma, label, hour, minute FROM horarylist;", new String[]{});
			cursor.moveToFirst();
			while(!cursor.isAfterLast()) {
				horarylist.add(new HoraryEntry(cursor.getInt(0), cursor.getString(1), cursor.getInt(2), cursor.getInt(3)));
				Log.d(this.getClass().getName() + ":" + "getHoraryList", "horaryentry: " + cursor.getInt(0) + " ; " + cursor.getString(1));
				cursor.moveToNext();
			}
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception(1)", e);
		} finally {
			try {
				if (cursor!=null) cursor.close();				
			} catch (Exception e2) {
				Log.e(this.getClass().getName() + whereami, "Exception(2)", e2);
			}
		}
		return horarylist;
	}
	
	/**
	 * Get the horarylist (get from BS servlet) and stores it into local storage.
	 * @param horarylist
	 * @return
	 */
	public boolean SaveLocalHoraryData(String[] horarylist) {
		String whereami = ":SaveLocalHoraryData";
		SQLiteStatement update = null;
		try {
			for(int i = 0; i<horarylist.length; i++) {
				Log.d(this.getClass().getName() + ":" + "SaveLocalHoraryData", horarylist[i]);
				String sqlsentence = "UPDATE horarylist SET hour=?, minute=? WHERE toma=?;";
				Log.d(this.getClass().getName() + ":" + "SaveLocalHoraryData", sqlsentence);
				update = this.database.compileStatement(sqlsentence);
				int hour = Integer.valueOf(horarylist[i].split(":")[0]);
				int minute = Integer.valueOf(horarylist[i].split(":")[1]);
				update.bindLong(1, hour);
				update.bindLong(2, minute);
				update.bindLong(3, i + 1);
				update.execute();
				update.close();
			}
		} catch (SQLiteException e) {
			Log.e(this.getClass().getName() + whereami, "SQLiteException", e);
			return false;
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
			return false;
		}
		return true;
	}
	
	/**
	 * Get one take item row (to show only one item in alarm).
	 * @param alarmRequestCode
	 * @return
	 */
	public TakeItem GetOneTakeItem(int alarmRequestCode) {
		String whereami = ":GetOneTakeItem";
		Cursor cursor = null;
		SQLiteStatement query = null;
		TakeItem takeitem = null;
		try {
			cursor = this.database.rawQuery("SELECT id, date, toma, product, quantity, status FROM takelist WHERE date=? AND toma=? AND status=0 ORDER BY id LIMIT 1;", new String[]{UIUtils.getCurrentDate(), String.valueOf(alarmRequestCode)});
			if(cursor!=null && cursor.moveToFirst()) {
				takeitem = new TakeItem(cursor.getLong(0), cursor.getString(1), cursor.getInt(2), cursor.getString(3), cursor.getInt(4), cursor.getInt(5));
				Log.i(this.getClass().getName() + whereami, takeitem.getProduct());
			} else {
				Log.i(this.getClass().getName() + whereami, "No alarm for this requestCode!");
			}
		} catch (Exception e) {
			Log.e(this.getClass().getName() + ":" + "GetOneTakeItem", "Exception", e);
		} finally {
			try {
				if(cursor!=null) { cursor.close(); cursor = null; }
				if(query!=null) { query.close(); query.clearBindings(); query = null; }				
			} catch (Exception e) {
				Log.e(this.getClass().getName() + whereami, "Exception(2)", e);
			}
		}
		return takeitem;
	}
	
	/**
	 * Finish a product take (called directly from Alarm).
	 * @param takeItemId id from the takelistitem row.
	 */
	public void FinishTakeItem(long takeItemId) {
		String whereami = ":FinishTakeItem";
		String sqlsentence = "";
		SQLiteStatement update = null;
		try {
			sqlsentence = "UPDATE takelist SET status=1 WHERE id=?;";
			Log.d(this.getClass().getName() + whereami, sqlsentence);
			update = this.database.compileStatement(sqlsentence);
			update.bindLong(1, takeItemId);
			update.execute();
		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		} finally {
			try {
				if(update!=null) { update.clearBindings(); update.close(); update = null; }				
			} catch (Exception e) {
				Log.e(this.getClass().getName() + whereami, "Exception", e);
			}
			
		}
	}

}
