package uoc.tfc.httpman;

import org.apache.http.client.CookieStore;
import org.apache.http.client.protocol.ClientContext;
import org.apache.http.protocol.HttpContext;
import org.json.JSONArray;
import org.json.JSONObject;

import uoc.tfc.database.Database;
import uoc.tfc.entities.User;
import uoc.tfc.task.GlobalAsyncTaskListener;
import android.content.Context;
import android.os.AsyncTask;
import android.util.Log;

/**
 * Class for managing http background process. The main objective is avoid
 * man thread hang.
 * 
 * @author omnizeus
 *
 */
public class HttpTaskMan extends AsyncTask<String, Integer, String> {

	// Constants
	public static String TASK_CHECK_BS_SERVICE_STATUS = "CheckBSServiceStatus";
	public static String TASK_GET_REMOTE_POSOLOGY_DATA = "GetRemotePosologyData";
	public static String TASK_LOGIN = "Login";
	public static String TASK_LOGOUT = "Logout";
	
	// private Context context;
	private GlobalAsyncTaskListener<String> listener;
	private HttpHelper httphelper;

	// Http management.
	private CookieStore cookieStore = null;
	private HttpContext httpContext = null;

	// Database
	private Database db = null;

	private String result;

	/**
	 * Constructor.
	 * 
	 * @param context
	 * @param cookiestore
	 * @param httpcontext
	 * @param listener
	 */
	public HttpTaskMan(Context context, CookieStore cookiestore,
			HttpContext httpcontext, GlobalAsyncTaskListener<String> listener) {
		super();
		String whereami = ":TaskMan";
		try {

			// Main listener for receiving TaskComplete event.
			this.listener = listener;

			// Main cookieStore & httpContext for http cookies/session
			// maintenance.
			this.cookieStore = cookiestore;
			this.httpContext = httpcontext;
			httpContext.setAttribute(ClientContext.COOKIE_STORE,
					this.cookieStore);

			// Internal Http Helper for http operations.
			httphelper = new HttpHelper(this.cookieStore, this.httpContext);

			// Prepare database object
			this.db = new Database(context.getApplicationContext());

		} catch (Exception e) {
			Log.e(this.getClass().getName() + whereami, "Exception", e);
		}

	}

	/**
	 * Pre Execute event: before executing task. Not used at the moment, but
	 * here for possible use.
	 */
	@Override
	protected void onPreExecute() {
		// TODO Auto-generated method stub
		super.onPreExecute();
	}

	/**
	 * Task execution (in background). Main process.
	 */
	@Override
	protected String doInBackground(String... params) {
		String whereami = ":doInBackground";
		String response = "";
		try {
			boolean res = false;
			// Sleep (for testing purposes).
			// Thread.sleep(1000);
			
			Log.i(this.getClass().getName() + whereami, params[0]);
			
			/**
			 * Task selection (by parameter).
			 */
			if (params[0].equals(HttpTaskMan.TASK_CHECK_BS_SERVICE_STATUS)) {

				/**
				 * Check BS service status.
				 */
				response = httphelper.check();

			} else if (params[0].equals(HttpTaskMan.TASK_LOGIN)) {

				/**
				 * Service login.
				 */
				if (params[1].equals("jose") && params[2].equals("jose")) {
					response = HttpHelper.RESP_SUCCESS_LOGIN_BYPASS;
				} else {
					JSONObject json = null;
					response = httphelper.login(params[1], params[2]);
					json = new JSONObject(response);
					res = json.getInt("statuscode") >= 0;
					response = (res ? HttpHelper.RESP_SUCCESS_LOGIN_CORRECT
							: HttpHelper.RESP_ERROR_LOGIN_INCORRECT);
				}

			} else if (params[0].equals(HttpTaskMan.TASK_GET_REMOTE_POSOLOGY_DATA)) {

				/**
				 * Get remote data. 
				 */
				
				JSONObject json = null;
				JSONArray jarrData = null;

				response = httphelper.clientbsGetpatientdata(params[1],
						params[2]);
				Log.d(this.getClass().getName() + whereami, "Response length: "
						+ response.length() + " ::: Data: " + response);
				json = new JSONObject(response);
				try {
					if (json.getInt("statuscode") <= 0) {
						response = HttpHelper.RESP_ERROR_PATIENT_DATA_ERR;
					}
				} catch (Exception e) {
					try {
						if (json.getString("result").equals("ok")) {
							jarrData = json.getJSONArray("takelist");
							response = HttpHelper.RESP_SUCCESS_PATIENT_DATA_OK;
						} else {
							response = HttpHelper.RESP_SUCCESS_PATIENT_NO_DATA;
						}
						// Store user data in local storage.
						db.StoreTakelist(jarrData);
						// Store service access data in local storage
						User user = new User(HttpHelper.BSSERVICE_NICKNAME,
								HttpHelper.BSSERVICE_PASSWORD, params[1],
								params[2]);
						db.StoreUser(user);

					} catch (Exception e2) {
						Log.e(this.getClass().getName() + whereami,
								"Exception", e);
						response = HttpHelper.RESP_EXCEPTION_UNHANDLED;
					}
				}

			} else if (params[0].equals(HttpTaskMan.TASK_LOGOUT)) {

				/**
				 * Service logout.
				 */
				
				if (httphelper.logout()) {
					response = HttpHelper.RESP_SUCCESS_LOGOUT;
				} else {
					response = HttpHelper.RESP_ERROR_LOGOUT;
				}

			}
			// Sleep (for testing purposes).
			// Thread.sleep(1000);
		} catch (Exception e) {
			Log.e(this.getClass().getName(), "Exception", e);
			response = HttpHelper.RESP_EXCEPTION_UNHANDLED;
		}
		// Internal result for onPostExecute access.
		this.result = response;
		// Return result
		return response;
	}

	/**
	 * On PostExecute event. Notify to listener the finish task.
	 */
	@Override
	protected void onPostExecute(String result) {
		// TODO Auto-generated method stub
		super.onPostExecute(result);
		// Toast toast = Toast.makeText(this.context, "Finalizado",
		// Toast.LENGTH_SHORT);
		// toast.setGravity(Gravity.CENTER, 0, 0);
		// toast.show();
		listener.onTaskComplete(this.result);

	}

	/**
	 * Process feedback.
	 */
	@Override
	protected void onProgressUpdate(Integer... values) {
		// TODO Auto-generated method stub
		super.onProgressUpdate(values);
	}

}
